/*
   rprintf.c

   a mixture of Uros Platise's mini rprintf.c and DJ Delorie's doprnt.c

	Adapted for Flash source --
	Devin Cook (dsc@oz.net)
	02-03-00

  Note:
  PUTC(ch) must be defined somewhere!
  %T or %t is used for Text (Rom Based Strings!)
*/

#include <stdarg.h>
#include <progmem.h>

#define SCRATCH 16
// #define RAM_STRING		0
// #define FLASH_STRING	(!RAM_STRING)

extern void PUTC(unsigned char ch);

char scratch[SCRATCH];

void printfs(const char* s)
{
  char ch;
  while ((ch = *s++)){PUTC(ch);}
}

void printfs_rom(const char* s)
{
  char ch;
  while ((ch = PRG_RDB(s++))){PUTC(ch);}
}

int rprintf(unsigned char Flash_Flag, const char *format, ...)
{
  char format_flag;
  unsigned int u_val, base;
  char *ptr;
  va_list ap;

  va_start (ap, format);
  for (;;)
  {
    while ((format_flag = (Flash_Flag ? (PRG_RDB(format++)) : (*format++))) != '%')
    {      /* Until '%' or '\0' */
      if (!format_flag){va_end (ap); return (0);}
      PUTC (format_flag);
    }

    switch (format_flag = (Flash_Flag ? (PRG_RDB(format++)) : (*format++)))
    {

    case 'c':
      format_flag = va_arg(ap,int);
    default:
      PUTC (format_flag);
      continue;
    case 'S':
    case 's':
      ptr = va_arg(ap,char *);
      printfs(ptr);
      continue;
    case 'T':
    case 't':
      ptr = va_arg(ap,char *);
      printfs_rom(ptr);
      continue;
    case 'o':
      base = 8;
      PUTC ('0');
      goto CONVERSION_LOOP;
    case 'd':
      base = 10;
      if (((int)u_val) < 0){
        u_val = - u_val;
        PUTC ('-');
      }
      goto CONVERSION_LOOP;
    case 'x':
      base = 16;

    CONVERSION_LOOP:
      u_val = va_arg(ap,int);
      ptr = scratch + SCRATCH;
      *--ptr = 0;
      do {
        char ch = u_val % base + '0';
        if (ch > '9')
          ch += 'a' - 10;
        *--ptr = ch;
        u_val /= base;
      } while (u_val);
      printfs(ptr);
    }
  }
}
